﻿using AutoMapper;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Localization;
using Microsoft.Extensions.Options;
using MS.Common.IDCode;
using MS.Component.Jwt;
using MS.Component.Jwt.UserClaim;
using MS.DbContexts;
using MS.Entities.AccountingService_Models;
using MS.Entities.AuthService_Models;
using MS.Models;
using MS.Models.ViewModel;
using MS.UnitOfWork;
using MS.WebCore;
using MS.WebCore.Core;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace MS.Services
{
    public class AccountService : BaseService, IAccountService
    {
        private readonly JwtService _jwtService;
        private readonly SiteSetting _siteSetting;
        bool ihostSyncRepLost = true;

        public AccountService(JwtService jwtService, IOptions<SiteSetting> options, IUnitOfWork<MSDbContext> unitOfWork, IMapper mapper, IdWorker idWorker, IClaimsAccessor claimsAccessor, IStringLocalizer localizer, IUnitOfWork<AccountingContext> AccountingContext, IUnitOfWork<SQLiteContext> SQLiteContext, IUnitOfWork<AuthContext> AuthContext) : base(unitOfWork, mapper, idWorker, claimsAccessor, localizer, AccountingContext, SQLiteContext, AuthContext)
        {
            _jwtService = jwtService;
            _siteSetting = options.Value;
        }
        public async Task<ExecuteResult<UserData>> Login(LoginViewModel viewModel)
        {
            var result = await viewModel.LoginValidate(_unitOfWork, _mapper, _siteSetting, _localizer);
            if (result.IsSucceed)
            {
                result.Result.Token = _jwtService.BuildToken(_jwtService.BuildClaims(result.Result));
                return new ExecuteResult<UserData>(result.Result);
            }
            else
            {
                return new ExecuteResult<UserData>(result.Message);
            }
        }
        public async Task<ExecuteResult<UserData>> syncFromVersion(long versionNo, bool excludingCurrentBu, string specificHeader)
        {
            ChangeHistory ch = new ChangeHistory();
            var SiteDevices = _AuthContext.GetRepository<SiteDevices>().GetAll()
                   .Where(a => a.HardwareIdentity == specificHeader).FirstOrDefault();
            var buid = SiteDevices.AsSubjectOfBusinessUnitId;
            var acct = _AccountingContext.GetRepository<BusinessUnitInfo>().GetAll().Where(a => a.Id == buid).FirstOrDefault();
            var CurrentBuId = acct.BusinessUnitInfoId;
            var originalBuId = acct.Id;
            if (versionNo == 0)
            {
                ch = _AccountingContext.GetRepository<ChangeHistory>().GetAll().OrderByDescending(a => a.TableName == -2).FirstOrDefault();
            }
            else
            {
                ch = _AccountingContext.GetRepository<ChangeHistory>().GetAll().OrderByDescending(a => a.Id).FirstOrDefault();
            }
            var lastModified = ch.LastModified;
            var currentVersionNo = ch.Id;

            if (versionNo == 0)
            {
                getAllRecordsToBeSynced(CurrentBuId, originalBuId, lastModified, excludingCurrentBu);
            }
            else
            {
                getRecordsGreaterThanVersionNo(CurrentBuId, originalBuId, versionNo, currentVersionNo, excludingCurrentBu);
            }

            return null;
        }
        public async Task<ChangeHistoryResponse> getAllRecordsToBeSynced(Guid? buid, Guid? originalBuId, DateTime? lastModified, bool excludingCurrentBu)
        {
            ChangeHistoryResponse changeHistoryResponse = new ChangeHistoryResponse();
            try
            {


                if (excludingCurrentBu)
                {
                    var dbCardInfo = await _AccountingContext.GetRepository<CardInfo>().FromSql(string.Format(@"
              select 
        cardinfo0_.id , 
        cardinfo0_.account_info_id, 
        cardinfo0_.account_id , 
        cardinfo0_.b_limit_car , 
        cardinfo0_.business_unit_info_id , 
        cardinfo0_.bu_info_created_id , 
        cardinfo0_.card_status , 
        cardinfo0_.ctc , 
        cardinfo0_.ctc_flag , 
        cardinfo0_.ctc_time , 
        cardinfo0_.card_class , 
        cardinfo0_.card_id , 
        cardinfo0_.card_no , 
        cardinfo0_.card_sno , 
        cardinfo0_.card_type , 
        cardinfo0_.car_license_no , 
        cardinfo0_.d_max_pay , 
        cardinfo0_.discount_no, 
        cardinfo0_.enable_sms , 
        cardinfo0_.gid , 
        cardinfo0_.holder , 
        cardinfo0_.points_awarded_total , 
        cardinfo0_.creation_date , 
        cardinfo0_.last_modified , 
        cardinfo0_.limit_times , 
        cardinfo0_.lmt_good , 
        cardinfo0_.lmt_oil , 
        cardinfo0_.m_max_pay , 
        cardinfo0_.balance, 
        cardinfo0_.once_max_pay , 
        cardinfo0_.operator_no, 
        cardinfo0_.expiration_date, 
        cardinfo0_.phone_no , 
        cardinfo0_.deposit_balance , 
        cardinfo0_.recharge_total, 
        cardinfo0_.sno, 
        cardinfo0_.activation_date , 
        cardinfo0_.tmac, 
        cardinfo0_.upload_flag , 
        cardinfo0_.user_no , 
        cardinfo0_.user_pin   
    from 
        card_info cardinfo0_  
    left outer join 
        business_unit_info businessun1_  
            on cardinfo0_.business_unit_info_id=businessun1_.id  
    left outer join 
        business_unit_info businessun2_  
            on cardinfo0_.bu_info_created_id=businessun2_.id  
    where 
        businessun1_.id= '{0}'
        and businessun2_.id<> '{1}'
    order by 
        cardinfo0_.last_modified asc 
", buid, originalBuId)).ToListAsync();
                    foreach (var card in dbCardInfo)
                    {
                        if (card.Holder != "已注销卡" && (card.LastModified != null && card.LastModified <= lastModified))
                        {
                            changeHistoryResponse.CardInfo.Add(card);
                        }
                    }


                    var dbAccountInfo = await _AccountingContext.GetRepository<AccountInfo>().FromSql(string.Format(@"
select
        accountinf0_.id,
        accountinf0_.account_id,
        accountinf0_.account_info_id,
        accountinf0_.account_no,
        accountinf0_.account_pin,
        accountinf0_.account_state,
        accountinf0_.account_type_id,
        accountinf0_.acct_sno,
        accountinf0_.acct_type,
        accountinf0_.address,
        accountinf0_.amount_limit,
        accountinf0_.balance,
        accountinf0_.belong_to,
        accountinf0_.business_unit_info_id,
        accountinf0_.bu_info_created_id,
        accountinf0_.card_class,
        accountinf0_.card_type,
        accountinf0_.cash,
        accountinf0_.certf_no,
        accountinf0_.certf_type,
        accountinf0_.creation_date,
        accountinf0_.daily_amount_limit,
        accountinf0_.discount_type,
        accountinf0_.enable_sms,
        accountinf0_.expiration_date,
        accountinf0_.fuel_no,
        accountinf0_.gid,
        accountinf0_.gas_liter_limit,
        accountinf0_.gift,
        accountinf0_.holder,
        accountinf0_.last_modified,
        accountinf0_.monthly_amount_limit,
        accountinf0_.offline_account_id,
        accountinf0_.offline_deposit_balance,
        accountinf0_.offline_phone_no,
        accountinf0_.operator_no,
        accountinf0_.physical_card_user_info,
        accountinf0_.pin_try_limit,
        accountinf0_.recharge_total,
        accountinf0_.sno,
        accountinf0_.t_mac,
        accountinf0_.upload_flag,
        accountinf0_.user_id,
        accountinf0_.user_info_id,
        accountinf0_.validation_date 
    from
        account_info accountinf0_ 
    left outer join
        business_unit_info businessun1_ 
            on accountinf0_.business_unit_info_id=businessun1_.id 
    left outer join
        business_unit_info businessun2_ 
            on accountinf0_.bu_info_created_id=businessun2_.id 
    where
        businessun1_.id= '{0}'
        and businessun2_.id<> '{1}'
    order by
        accountinf0_.last_modified asc
", buid, originalBuId)).ToListAsync();
                    foreach (var accountInfo in dbAccountInfo)
                    {
                        if ((accountInfo.BelongTo != null && accountInfo.BelongTo != "已注销账户") && (accountInfo.LastModified != null && accountInfo.LastModified <= lastModified))
                        {
                            changeHistoryResponse.AccountInfo.Add(accountInfo);
                        }
                    }

                    var dbBlackCardInfo = await _AccountingContext.GetRepository<BlackCardInfo>().FromSql(string.Format(@"
                  select
        blackcardi0_.id,
        blackcardi0_.acct_gid,
        blackcardi0_.acct_id,
        blackcardi0_.black_card_type,
        blackcardi0_.black_date,
        blackcardi0_.bu_info_id,
        blackcardi0_.bu_info_created_id,
        blackcardi0_.card_no,
        blackcardi0_.card_type,
        blackcardi0_.discount_no,
        blackcardi0_.gid,
        blackcardi0_.is_deleted,
        blackcardi0_.last_modified,
        blackcardi0_.reason,
        blackcardi0_.upload_flag
    from
        black_card_info blackcardi0_ 
    left outer join
        business_unit_info businessun1_ 
            on blackcardi0_.bu_info_id=businessun1_.id 
    left outer join
        business_unit_info businessun2_ 
            on blackcardi0_.bu_info_created_id=businessun2_.id 
    where
        businessun1_.id= '{0}'
        and businessun2_.id<> '{1}'
    order by
        blackcardi0_.last_modified asc
", buid, originalBuId)).ToListAsync();
                    foreach (var black in dbBlackCardInfo)
                    {
                        if (black.LastModified <= lastModified)
                        {
                            changeHistoryResponse.BlackCardInfo.Add(black);
                        }
                    }

                    var dbGrayInfo = await _AccountingContext.GetRepository<GrayInfo>().FromSql(string.Format(@"
                 select
        grayinfo0_.id,
        grayinfo0_.bu_info_id,
        grayinfo0_.bu_info_created_id,
        grayinfo0_.card_balance,
        grayinfo0_.card_no,
        grayinfo0_.comm_id,
        grayinfo0_.ctc,
        grayinfo0_.discount_no,
        grayinfo0_.end_pump,
        grayinfo0_.gid,
        grayinfo0_.gmac,
        grayinfo0_.is_deleted,
        grayinfo0_.last_modified,
        grayinfo0_.money,
        grayinfo0_.nozzle_no,
        grayinfo0_.pay_terminal_id,
        grayinfo0_.paymode_id,
        grayinfo0_.price,
        grayinfo0_.psam_asn,
        grayinfo0_.psam_tac,
        grayinfo0_.psam_tid,
        grayinfo0_.psam_ttc,
        grayinfo0_.pump_no,
        grayinfo0_.pump_type,
        grayinfo0_.real_money,
        grayinfo0_.sno,
        grayinfo0_.sequence_no,
        grayinfo0_.tac,
        grayinfo0_.tmac,
        grayinfo0_.trd_type,
        grayinfo0_.ttc,
        grayinfo0_.ttc_time,
        grayinfo0_.ttc_time_end,
        grayinfo0_.upload_flag,
        grayinfo0_.volume 
    from
        gray_info grayinfo0_ 
    left outer join
        business_unit_info businessun1_ 
            on grayinfo0_.bu_info_id=businessun1_.id 
    left outer join
        business_unit_info businessun2_ 
            on grayinfo0_.bu_info_created_id=businessun2_.id 
    where
        businessun1_.id= '{0}'
        and businessun2_.id<> '{1}'
    order by
        grayinfo0_.last_modified asc

", buid, originalBuId)).ToListAsync();
                    foreach (var grayInfo in dbGrayInfo)
                    {
                        if (grayInfo.LastModified <= lastModified)
                        {
                            changeHistoryResponse.GrayInfo.Add(grayInfo);
                        }
                    }

                    if (ihostSyncRepLost)
                    {
                        var dbRepLossInfo = await _AccountingContext.GetRepository<CardRepLossInfo>().FromSql(string.Format(@"
select
        cardreplos0_.id,
        cardreplos0_.bu_info_id,
        cardreplos0_.bu_info_created_id,
        cardreplos0_.card_no,
        cardreplos0_.gid,
        cardreplos0_.last_modified,
        cardreplos0_.loss_time,
        cardreplos0_.operation_type,
        cardreplos0_.operator_no,
        cardreplos0_.reason,
        cardreplos0_.sno 
    from
        card_rep_loss_info cardreplos0_ 
    left outer join
        business_unit_info businessun1_ 
            on cardreplos0_.bu_info_id=businessun1_.id 
    left outer join
        business_unit_info businessun2_ 
            on cardreplos0_.bu_info_created_id=businessun2_.id 
    where
        businessun1_.id= '{0}'
        and businessun2_.id<> '{1}'
    order by
        cardreplos0_.last_modified asc
", buid, originalBuId)).ToListAsync();
                        foreach (var lossInfo in dbRepLossInfo)
                        {
                            if (lossInfo.LastModified <= lastModified)
                            {
                                changeHistoryResponse.cardRepLossInfos.Add(lossInfo);
                            }
                        }
                    }

                }
                else
                {
                    var dbCardInfo = _AccountingContext.GetRepository<CardInfo>().GetAll().Where(a => a.BusinessUnitInfoId == buid).ToList();
                    //var dbCardInfo = await _AccountingContext.GetRepository<CardInfo>().FromSql(sql).ToListAsync();

                    foreach (var card in dbCardInfo)
                    {
                        if (card.Holder != "已注销卡" && (card.LastModified != null && card.LastModified <= lastModified))
                        {
                            changeHistoryResponse.CardInfo.Add(card);
                        }
                    }

//                    var dbAccountInfo = await _AccountingContext.GetRepository<AccountInfo>().FromSql(string.Format(@"
//                 select accountinf0_.*
//    from
//        account_info accountinf0_ 
//    left outer join
//        business_unit_info businessun1_ 
//            on accountinf0_.business_unit_info_id=businessun1_.id 
//    where
//        businessun1_.id= '{0}'
//    order by
//        accountinf0_.last_modified asc
//", buid)).ToListAsync();

                    var dbAccountInfo = _AccountingContext.GetRepository<AccountInfo>().GetAll().Where(a => a.BusinessUnitInfoId == buid).ToList();
                    foreach (var accountInfo in dbAccountInfo)
                    {
                        if ((accountInfo.BelongTo != null && accountInfo.BelongTo != "已注销账户") && (accountInfo.LastModified != null && accountInfo.LastModified <= lastModified))
                        {
                            changeHistoryResponse.AccountInfo.Add(accountInfo);
                        }
                    }

                    var dbBlackCardInfo = _AccountingContext.GetRepository<BlackCardInfo>().GetAll().Where(a => a.BuInfoId == buid).ToList();
//                    var dbBlackCardInfo = await _AccountingContext.GetRepository<BlackCardInfo>().FromSql(string.Format(@"
//                select
//        blackcardi0_.id,
//        blackcardi0_.acct_gid,
//        blackcardi0_.acct_id,
//        blackcardi0_.black_card_type,
//        blackcardi0_.black_date,
//        blackcardi0_.bu_info_id,
//        blackcardi0_.bu_info_created_id,
//        blackcardi0_.card_no,
//        blackcardi0_.card_type,
//        blackcardi0_.discount_no,
//        blackcardi0_.gid,
//        blackcardi0_.is_deleted,
//        blackcardi0_.last_modified,
//        blackcardi0_.reason,
//        blackcardi0_.upload_flag 
//    from
//        black_card_info blackcardi0_ 
//    left outer join
//        business_unit_info businessun1_ 
//            on blackcardi0_.bu_info_id=businessun1_.id 
//    where
//        businessun1_.id= '{0}'
//    order by
//        blackcardi0_.last_modified asc
//", buid)).ToListAsync();
                    foreach (var black in dbBlackCardInfo)
                    {
                        if (black.LastModified <= lastModified)
                        {
                            changeHistoryResponse.BlackCardInfo.Add(black);
                        }
                    }
                    var dbGrayInfo = _AccountingContext.GetRepository<GrayInfo>().GetAll().Where(a => a.BuInfoId == buid).ToList();
//                    var dbGrayInfo = await _AccountingContext.GetRepository<GrayInfo>().FromSql(string.Format(@"
//                   select
//        grayinfo0_.id,
//        grayinfo0_.bu_info_id,
//        grayinfo0_.bu_info_created_id,
//        grayinfo0_.card_balance,
//        grayinfo0_.card_no,
//        grayinfo0_.comm_id,
//        grayinfo0_.ctc,
//        grayinfo0_.discount_no,
//        grayinfo0_.end_pump,
//        grayinfo0_.gid,
//        grayinfo0_.gmac,
//        grayinfo0_.is_deleted,
//        grayinfo0_.last_modified,
//        grayinfo0_.money,
//        grayinfo0_.nozzle_no,
//        grayinfo0_.pay_terminal_id,
//        grayinfo0_.paymode_id,
//        grayinfo0_.price,
//        grayinfo0_.psam_asn,
//        grayinfo0_.psam_tac,
//        grayinfo0_.psam_tid,
//        grayinfo0_.psam_ttc,
//        grayinfo0_.pump_no,
//        grayinfo0_.pump_type,
//        grayinfo0_.real_money,
//        grayinfo0_.sno,
//        grayinfo0_.sequence_no,
//        grayinfo0_.tac,
//        grayinfo0_.tmac,
//        grayinfo0_.trd_type,
//        grayinfo0_.ttc,
//        grayinfo0_.ttc_time,
//        grayinfo0_.ttc_time_end,
//        grayinfo0_.upload_flag,
//        grayinfo0_.volume
//    from
//        gray_info grayinfo0_ 
//    left outer join
//        business_unit_info businessun1_ 
//            on grayinfo0_.bu_info_id=businessun1_.id 
//    where
//        businessun1_.id= '{0}'
//    order by
//        grayinfo0_.last_modified asc
//", buid)).ToListAsync();
                    foreach (var grayInfo in dbGrayInfo)
                    {
                        if (grayInfo.LastModified <= lastModified)
                        {
                            changeHistoryResponse.GrayInfo.Add(grayInfo);
                        }
                    }

                    if (ihostSyncRepLost)
                    {
                        var dbRepLossInfo = _AccountingContext.GetRepository<CardRepLossInfo>().GetAll().Where(a => a.BuInfoId == buid).ToList();
//                        var dbRepLossInfo = await _AccountingContext.GetRepository<CardRepLossInfo>().FromSql(string.Format(@"
//select
//        cardreplos0_.id,
//        cardreplos0_.bu_info_id,
//        cardreplos0_.bu_info_created_id,
//        cardreplos0_.card_no,
//        cardreplos0_.gid,
//        cardreplos0_.last_modified,
//        cardreplos0_.loss_time,
//        cardreplos0_.operation_type,
//        cardreplos0_.operator_no,
//        cardreplos0_.reason,
//        cardreplos0_.sno 
//    from
//        card_rep_loss_info cardreplos0_ 
//    left outer join
//        business_unit_info businessun1_ 
//            on cardreplos0_.bu_info_id=businessun1_.id 
//    where
//        businessun1_.id= '{0}'
//    order by
//        cardreplos0_.last_modified asc
//", buid)).ToListAsync();
                        foreach (var lossInfo in dbRepLossInfo)
                        {
                            if (lossInfo.LastModified <= lastModified)
                            {
                                changeHistoryResponse.cardRepLossInfos.Add(lossInfo);
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {

            }
            return changeHistoryResponse;
        }
        public Task getRecordsGreaterThanVersionNo(Guid? buid, Guid? originalBuId, long fromVersionNo, long toVersionNo, bool excludingCurrentBu)
        {
            List<ChangeHistory> changeHistoryResponse = new List<ChangeHistory>();
            ChangeHistoryResponse response = new ChangeHistoryResponse();
            if (excludingCurrentBu)
            {
                changeHistoryResponse = _AccountingContext.GetRepository<ChangeHistory>().FromSql(string.Format(@"
                select 
        changehist0_.id, 
        changehist0_.bu_id , 
        changehist0_.bu_info_created_id , 
        changehist0_.last_modified , 
        changehist0_.record_id , 
        changehist0_.table_name  
    from 
        change_history changehist0_  
    where 
        changehist0_.bu_id= '{0}'
        and changehist0_.bu_info_created_id <> '{1}'
        and changehist0_.id>  '{2}'
        and changehist0_.id<=  '{3}' 
    order by 
        changehist0_.id asc 
", buid, originalBuId, fromVersionNo, toVersionNo)).ToListAsync().Result;
            }
            else
            {
                changeHistoryResponse = _AccountingContext.GetRepository<ChangeHistory>().FromSql(string.Format(@"
                 select 
        changehist0_.id , 
        changehist0_.bu_id, 
        changehist0_.bu_info_created_id , 
        changehist0_.last_modified , 
        changehist0_.record_id , 
        changehist0_.table_name   
    from 
        change_history changehist0_  
    where 
        changehist0_.bu_id='{0}' 
        and changehist0_.id>'{1}' 
        and changehist0_.id<='{2}' 
    order by 
        changehist0_.id asc 
", buid, fromVersionNo, toVersionNo)).ToListAsync().Result;
            }
            var changes = removeDuplicatedRecords(changeHistoryResponse);
            foreach (var changeHistory in changes)
            {
                var tableName = changeHistory.TableName;
                var recordId = changeHistory.RecordId;

                if (tableName == 0)//卡
                {
                    var card = _AccountingContext.GetRepository<CardInfo>().Find(recordId);
                    if (card != null)
                    {
                        if (card.Holder != "已注销卡")
                        {
                            response.CardInfo.Add(card);
                        }
                    }
                }
                else if (tableName == 1)//ACCOUNT_INFO
                {
                    var account = _AccountingContext.GetRepository<AccountInfo>().Find(recordId);
                    if (account != null)
                    {
                        if (account.BelongTo != "已注销账户")
                        {
                            response.AccountInfo.Add(account);
                        }
                    }
                }
                else if (tableName == 2)
                {
                    var black = _AccountingContext.GetRepository<BlackCardInfo>().Find(recordId);
                    if (black != null)
                    {
                        response.BlackCardInfo.Add(black);
                    }
                }
                else if (tableName == 3)
                {
                    var gray = _AccountingContext.GetRepository<GrayInfo>().Find(recordId);
                    if (gray != null)
                    {
                        response.GrayInfo.Add(gray);
                    }
                }
                else if (tableName == 4)
                {
                    var loss = _AccountingContext.GetRepository<CardRepLossInfo>().Find(recordId);
                    if (loss != null)
                    {
                        response.cardRepLossInfos.Add(loss);
                    }
                }
                else
                {
                    if (tableName == -2 || tableName == -1)
                    {
                        MS.WebCore.Logger.Logger.WriteLog
                            ("Ignored table typ：" + tableName);
                    }
                    else
                    {
                        MS.WebCore.Logger.Logger.WriteLog("Unexpected table type " + tableName);
                    }
                }
            }
            return null;
        }
        public List<ChangeHistory> removeDuplicatedRecords(List<ChangeHistory> changeHistories)
        {
            var changes = new List<ChangeHistory>();
            var cardAndAccountList = new List<ChangeHistory>();
            var otherInfoList = new List<ChangeHistory>();
            foreach (var change in changeHistories)
            {
                if (change.TableName == 1 || change.TableName == 0)
                {
                    cardAndAccountList.Add(change);
                }
                else
                {
                    otherInfoList.Add(change);
                }
            }
            cardAndAccountList = RemoveDuplicatesAndPreserveOrder(cardAndAccountList);
            changes.AddRange(cardAndAccountList);
            changes.AddRange(otherInfoList);
            return changeHistories;
        }
        static List<ChangeHistory> RemoveDuplicatesAndPreserveOrder(List<ChangeHistory> sourceList)
        {
            HashSet<ChangeHistory> uniqueSet = new HashSet<ChangeHistory>(sourceList);

            // 使用 LINQ 的 OrderBy 方法按原列表中元素的出现顺序排列
            return sourceList.OrderBy(item => sourceList.IndexOf(item))
                             .Where(item => uniqueSet.Contains(item))
                             .ToList();
        }
    }
}
